@@ -500,11 +500,6 @@ public class Util {
         else                return s;
     }
 
-    /**
-     * Write-only buffer.
-     */
-    private static final byte[] garbage = new byte[8192];
-
     /**
      * Computes MD5 digest of the given input stream.
      *
@@ -517,9 +512,10 @@ public class Util {
         try {
             MessageDigest md5 = MessageDigest.getInstance("MD5");
 
+            byte[] buffer = new byte[1024];
             DigestInputStream in =new DigestInputStream(source,md5);
             try {
-                while(in.read(garbage)>0)
+                while(in.read(buffer)>0)
                     ; // simply discard the input
             } finally {
                 in.close();
@@ -699,7 +695,7 @@ public class Util {
             OutputStreamWriter w = new OutputStreamWriter(buf,"UTF-8");
 
             for (int i = 0; i < s.length(); i++) {
-                int c = (int) s.charAt(i);
+                int c = s.charAt(i);
                 if (c<128 && c!=' ') {
                     out.append((char) c);
                 } else {
