@@ -1,7 +1,7 @@
 /*
  * The MIT License
  *
- * Copyright (c) 2009-2010, Sun Microsystems, Inc.
+ * Copyright (c) 2009-2010, Sun Microsystems, Inc., CloudBees, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,54 +23,53 @@
  */
 package hudson.tools;
 
+import com.gargoylesoftware.htmlunit.ElementNotFoundException;
+import com.gargoylesoftware.htmlunit.Page;
+import com.gargoylesoftware.htmlunit.WebClient;
+import com.gargoylesoftware.htmlunit.html.HtmlForm;
+import com.gargoylesoftware.htmlunit.html.HtmlPage;
 import hudson.AbortException;
 import hudson.Extension;
 import hudson.FilePath;
-import hudson.ProxyConfiguration;
-import hudson.Util;
 import hudson.Launcher;
-import hudson.model.Hudson;
-import hudson.util.FormValidation;
-import hudson.util.ArgumentListBuilder;
-import hudson.util.IOException2;
-import hudson.model.Node;
-import hudson.model.TaskListener;
+import hudson.Launcher.ProcStarter;
+import hudson.Util;
 import hudson.model.DownloadService.Downloadable;
 import hudson.model.JDK;
-import static hudson.tools.JDKInstaller.Preference.*;
+import hudson.model.Node;
+import hudson.model.TaskListener;
 import hudson.remoting.Callable;
-import org.jvnet.robust_http_client.RetryableHttpStream;
-import org.kohsuke.stapler.DataBoundConstructor;
-import org.kohsuke.stapler.QueryParameter;
+import hudson.util.ArgumentListBuilder;
+import hudson.util.FormValidation;
+import hudson.util.HttpResponses;
+import hudson.util.Secret;
+import hudson.model.Hudson;
+import net.sf.json.JSONObject;
 import org.apache.commons.io.IOUtils;
-import org.apache.commons.io.output.NullWriter;
-import org.w3c.tidy.Tidy;
-import org.dom4j.io.DOMReader;
-import org.dom4j.Document;
-import org.dom4j.Element;
+import org.kohsuke.stapler.DataBoundConstructor;
+import org.kohsuke.stapler.HttpResponse;
+import org.kohsuke.stapler.QueryParameter;
+import org.kohsuke.stapler.Stapler;
 
+import javax.servlet.ServletException;
 import java.io.ByteArrayInputStream;
+import java.io.DataInputStream;
 import java.io.File;
 import java.io.FileOutputStream;
 import java.io.IOException;
-import java.io.PrintStream;
+import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.OutputStreamWriter;
-import java.io.PrintWriter;
-import java.io.InputStream;
+import java.io.PrintStream;
 import java.net.URL;
-import java.net.HttpURLConnection;
-import java.net.URLEncoder;
 import java.util.ArrayList;
-import java.util.List;
-import java.util.Locale;
 import java.util.Arrays;
 import java.util.Iterator;
+import java.util.List;
+import java.util.Locale;
 import java.util.logging.Logger;
-import java.util.regex.Matcher;
-import java.util.regex.Pattern;
 
-import net.sf.json.JSONObject;
+import static hudson.tools.JDKInstaller.Preference.*;
 
 /**
  * Install JDKs from java.sun.com.
@@ -120,7 +119,7 @@ public class JDKInstaller extends ToolInstaller {
             Platform p = Platform.of(node);
             URL url = locate(log, p, CPU.of(node));
 
-            out.println("Downloading "+url);
+//            out.println("Downloading "+url);
             FilePath file = expectedLocation.child(p.bundleFileName);
             file.copyFrom(url);
 
@@ -161,8 +160,24 @@ public class JDKInstaller extends ToolInstaller {
         switch (p) {
         case LINUX:
         case SOLARIS:
-            fs.chmod(jdkBundle,0755);
-            int exit = launcher.launch().cmds(jdkBundle, "-noregister")
+            // JDK on Unix up to 6 was distributed as shell script installer, but in JDK7 it switched to a plain tgz.
+            // so check if the file is gzipped, and if so, treat it accordingly
+            byte[] header = new byte[2];
+            {
+                DataInputStream in = new DataInputStream(fs.read(jdkBundle));
+                in.readFully(header);
+                in.close();
+            }
+
+            ProcStarter starter;
+            if (header[0]==0x1F && header[1]==(byte)0x8B) {// gzip
+                starter = launcher.launch().cmds("tar", "xvzf", jdkBundle);
+            } else {
+                fs.chmod(jdkBundle,0755);
+                starter = launcher.launch().cmds(jdkBundle, "-noregister");
+            }
+
+            int exit = starter
                     .stdin(new ByteArrayInputStream("yes".getBytes())).stdout(out)
                     .pwd(new FilePath(launcher.getChannel(), expectedLocation)).join();
             if (exit != 0)
@@ -295,9 +310,91 @@ public class JDKInstaller extends ToolInstaller {
         File cache = getLocalCacheFile(platform, cpu);
         if (cache.exists()) return cache.toURL();
 
-        HttpURLConnection con = locateStage1(platform, cpu);
-        String page = IOUtils.toString(con.getInputStream());
-        URL src = locateStage2(log, page);
+        log.getLogger().println("Installing JDK "+id);
+        JDKFamilyList families = JDKList.all().get(JDKList.class).toList();
+        if (families.isEmpty())
+            throw new IOException("JDK data is empty.");
+
+        JDKRelease release = families.getRelease(id);
+        if (release==null)
+            throw new IOException("Unable to find JDK with ID="+id);
+
+        JDKFile primary=null,secondary=null;
+        for (JDKFile f : release.files) {
+            String vcap = f.name.toUpperCase(Locale.ENGLISH);
+
+            // JDK files have either 'windows', 'linux', or 'solaris' in its name, so that allows us to throw
+            // away unapplicable stuff right away
+            if(!platform.is(vcap))
+                continue;
+
+            switch (cpu.accept(vcap)) {
+            case PRIMARY:   primary = f;break;
+            case SECONDARY: secondary=f;break;
+            case UNACCEPTABLE:  break;
+            }
+        }
+
+        if(primary==null)   primary=secondary;
+        if(primary==null)
+            throw new AbortException("Couldn't find the right download for "+platform+" and "+ cpu +" combination");
+        LOGGER.fine("Platform choice:"+primary);
+
+        log.getLogger().println("Downloading JDK from "+primary.filepath);
+        URL src = new URL(primary.filepath);
+
+        WebClient wc = new WebClient();
+        wc.setJavaScriptEnabled(false);
+        wc.setCssEnabled(false);
+        Page page = wc.getPage(src);
+        int authCount=0;
+        int totalPageCount=0;
+        while (page instanceof HtmlPage) {
+            // some times we are redirected to the SSO login page.
+            HtmlPage html = (HtmlPage) page;
+            URL loginUrl = page.getWebResponse().getUrl();
+            if (!loginUrl.getHost().equals("login.oracle.com"))
+                throw new IOException("Expected to see a login page but instead saw "+loginUrl);
+
+            String u = getDescriptor().getUsername();
+            Secret p = getDescriptor().getPassword();
+            if (u==null || p==null) {
+                log.hyperlink(getCredentialPageUrl(),"Oracle now requires Oracle account to download previous versions of JDK. Please specify your Oracle account username/password.\n");
+                throw new AbortException("Unable to install JDK unless a valid username/password is provided.");
+            }
+
+            if (totalPageCount++>16) // looping too much
+                throw new IOException("Unable to find the login form in "+html.asXml());
+
+            try {
+                // JavaScript check page. Just submit and move on
+                HtmlForm loginForm = html.getFormByName("myForm");
+                page = loginForm.submit(null);
+                continue;
+            } catch (ElementNotFoundException e) {
+                // fall through
+            }
+
+            try {
+                // real authentication page
+                if (authCount++ > 3) {
+                    log.hyperlink(getCredentialPageUrl(),"Your Oracle account doesn't appear valid. Please specify a valid username/password\n");
+                    throw new AbortException("Unable to install JDK unless a valid username/password is provided.");
+                }
+                HtmlForm loginForm = html.getFormByName("LoginForm");
+                loginForm.getInputByName("ssousername").setValueAttribute(u);
+                loginForm.getInputByName("password").setValueAttribute(p.getPlainText());
+                page = loginForm.submit(null);
+                continue;
+            } catch (ElementNotFoundException e) {
+                // fall through
+            }
+
+            throw new IOException("Unable to find the login form in "+html.asXml());
+        }
+
+        // TODO: there's awful inefficiency in htmlunit where it loads the whole binary into one big byte array.
+        // needs to modify it to use temporary file or something
 
         // download to a temporary file and rename it in to handle concurrency and failure correctly,
         File tmp = new File(cache.getPath()+".tmp");
@@ -305,14 +402,7 @@ public class JDKInstaller extends ToolInstaller {
         try {
             FileOutputStream out = new FileOutputStream(tmp);
             try {
-                IOUtils.copy(new RetryableHttpStream(src) {
-                    @Override
-                    protected HttpURLConnection connect() throws IOException {
-                        HttpURLConnection con = (HttpURLConnection) ProxyConfiguration.open(url);
-                        con.setReadTimeout(60*1000);    // don't block forever, but don't let the slow client fail with false positives either
-                        return con;
-                    }
-                }, out);
+                IOUtils.copy(page.getWebResponse().getContentAsStream(), out);
             } finally {
                 out.close();
             }
@@ -324,115 +414,8 @@ public class JDKInstaller extends ToolInstaller {
         }
     }
 
-    @SuppressWarnings("unchecked") // dom4j doesn't do generics, apparently... should probably switch to XOM
-    private HttpURLConnection locateStage1(Platform platform, CPU cpu) throws IOException {
-        URL url = new URL("https://cds.sun.com/is-bin/INTERSHOP.enfinity/WFS/CDS-CDS_Developer-Site/en_US/-/USD/ViewProductDetail-Start?ProductRef="+id);
-        String cookie;
-        Element form;
-        try {
-            HttpURLConnection con = (HttpURLConnection) ProxyConfiguration.open(url);
-            cookie = con.getHeaderField("Set-Cookie");
-            LOGGER.fine("Cookie="+cookie);
-
-            Tidy tidy = new Tidy();
-            tidy.setErrout(new PrintWriter(new NullWriter()));
-            DOMReader domReader = new DOMReader();
-            Document dom = domReader.read(tidy.parseDOM(con.getInputStream(), null));
-
-            form = null;
-            for (Element e : (List<Element>)dom.selectNodes("//form")) {
-                String action = e.attributeValue("action");
-                LOGGER.fine("Found form:"+action);
-                if(action.contains("ViewFilteredProducts")) {
-                    form = e;
-                    break;
-                }
-            }
-        } catch (IOException e) {
-            throw new IOException2("Failed to access "+url,e);
-        }
-
-        url = new URL(form.attributeValue("action"));
-        try {
-            HttpURLConnection con = (HttpURLConnection) ProxyConfiguration.open(url);
-            con.setRequestMethod("POST");
-            con.setDoOutput(true);
-            con.setRequestProperty("Cookie",cookie);
-            con.setRequestProperty("Content-Type","application/x-www-form-urlencoded");
-            PrintStream os = new PrintStream(con.getOutputStream());
-
-            // select platform
-            String primary=null,secondary=null;
-            Element p = (Element)form.selectSingleNode(".//select[@id='dnld_platform']");
-            for (Element opt : (List<Element>)p.elements("option")) {
-                String value = opt.attributeValue("value");
-                String vcap = value.toUpperCase(Locale.ENGLISH);
-                if(!platform.is(vcap))  continue;
-                switch (cpu.accept(vcap)) {
-                case PRIMARY:   primary = value;break;
-                case SECONDARY: secondary=value;break;
-                case UNACCEPTABLE:  break;
-                }
-            }
-            if(primary==null)   primary=secondary;
-            if(primary==null)
-            throw new AbortException("Couldn't find the right download for "+platform+" and "+ cpu +" combination");
-            os.print(p.attributeValue("name")+'='+primary);
-            LOGGER.fine("Platform choice:"+primary);
-
-            // select language
-            Element l = (Element)form.selectSingleNode(".//select[@id='dnld_language']");
-            if (l != null) {
-                os.print("&"+l.attributeValue("name")+"="+l.element("option").attributeValue("value"));
-            }
-
-            // the rest
-            for (Element e : (List<Element>)form.selectNodes(".//input")) {
-                os.print('&');
-                os.print(e.attributeValue("name"));
-                os.print('=');
-                String value = e.attributeValue("value");
-                if(value==null)
-                    os.print("on"); // assume this is a checkbox
-                else
-                    os.print(URLEncoder.encode(value,"UTF-8"));
-            }
-            os.close();
-            return con;
-        } catch (IOException e) {
-            throw new IOException2("Failed to access "+url,e);
-        }
-    }
-
-    private URL locateStage2(TaskListener log, String page) throws IOException {
-        Pattern HREF = Pattern.compile("<a href=\"(http://cds.sun.com/[^\"]+/VerifyItem-Start[^\"]+)\"");
-        Matcher m = HREF.matcher(page);
-        // this page contains a missing --> that confuses dom4j/jtidy
-
-        log.getLogger().println("Choosing the download bundle");
-        List<String> urls = new ArrayList<String>();
-
-        while(m.find()) {
-            String url = m.group(1);
-            LOGGER.fine("Considering a download link:"+ url);
-
-            // still more options to choose from.
-            // avoid rpm bundles, and avoid tar.Z bundle
-            if(url.contains("rpm"))  continue;
-            if(url.contains("tar.Z"))  continue;
-            // sparcv9 bundle is add-on to the sparc bundle, so just download 32bit sparc bundle, even on 64bit system
-            if(url.contains("sparcv9"))  continue;
-
-            urls.add(url);
-            LOGGER.fine("Found a download candidate: "+ url);
-        }
-
-        if (urls.isEmpty()) {
-            throw new IOException("found no matches in: " + page);
-        }
-
-        // prefer the first match because sometimes "optional downloads" follow the main bundle
-        return new URL(urls.get(0));
+    private String getCredentialPageUrl() {
+        return "/"+getDescriptor().getDescriptorUrl()+"/enterCredential";
     }
 
     public enum Preference {
@@ -492,15 +475,15 @@ public class JDKInstaller extends ToolInstaller {
             switch (this) {
             // these two guys are totally incompatible with everything else, so no fallback
             case Sparc:     return must(line.contains("SPARC"));
-            case Itanium:   return must(line.contains("ITANIUM"));
+            case Itanium:   return must(line.contains("IA64"));
 
             // 64bit Solaris, Linux, and Windows can all run 32bit executable, so fall back to 32bit if 64bit bundle is not found
             case amd64:
+                if(line.contains("SPARC") || line.contains("IA64"))  return UNACCEPTABLE;
                 if(line.contains("64"))     return PRIMARY;
-                if(line.contains("SPARC") || line.contains("ITANIUM"))  return UNACCEPTABLE;
                 return SECONDARY;
             case i386:
-                if(line.contains("64") || line.contains("SPARC") || line.contains("ITANIUM"))     return UNACCEPTABLE;
+                if(line.contains("64") || line.contains("SPARC") || line.contains("IA64"))     return UNACCEPTABLE;
                 return PRIMARY;
             }
             return UNACCEPTABLE;
@@ -546,24 +529,73 @@ public class JDKInstaller extends ToolInstaller {
     }
 
     public static final class JDKFamilyList {
-        public JDKFamily[] jdks = new JDKFamily[0];
+        public int version;
+        public JDKFamily[] data = new JDKFamily[0];
+
+        public boolean isEmpty() {
+            for (JDKFamily f : data) {
+                if (f.releases.length>0)
+                    return false;
+            }
+            return true;
+        }
+
+        public JDKRelease getRelease(String productCode) {
+            for (JDKFamily f : data) {
+                for (JDKRelease r : f.releases) {
+                    if (r.matchesId(productCode))
+                        return r;
+                }
+            }
+            return null;
+        }
     }
 
     public static final class JDKFamily {
         public String name;
-        public InstallableJDK[] list;
+        public JDKRelease[] releases;
     }
 
-    public static final class InstallableJDK {
+    public static final class JDKRelease {
+        /**
+         * This maps to the former product code, like "jdk-6u13-oth-JPR"
+         */
         public String name;
         /**
-         * Product code.
+         * This is human readable.
          */
-        public String id;
+        public String title;
+        public JDKFile[] files;
+
+        /**
+         * We used to use IDs like "jdk-6u13-oth-JPR@CDS-CDS_Developer", but Oracle switched to just "jdk-6u13-oth-JPR".
+         * This method matches if the specified string matches the name, and it accepts both the old and the new format.
+         */
+        public boolean matchesId(String rhs) {
+            return rhs!=null && (rhs.equals(name) || rhs.startsWith(name+"@"));
+        }
+    }
+
+    public static final class JDKFile {
+        public String name;
+        public String title;
+        public String filepath;
+    }
+
+    @Override
+    public DescriptorImpl getDescriptor() {
+        return (DescriptorImpl)super.getDescriptor();
     }
 
     @Extension
     public static final class DescriptorImpl extends ToolInstallerDescriptor<JDKInstaller> {
+        private String username;
+        private Secret password;
+
+        public DescriptorImpl() {
+            load();
+        }
+
         public String getDisplayName() {
             return Messages.JDKInstaller_DescriptorImpl_displayName();
         }
@@ -573,13 +605,18 @@ public class JDKInstaller extends ToolInstaller {
             return toolType==JDK.class;
         }
 
+        public String getUsername() {
+            return username;
+        }
+
+        public Secret getPassword() {
+            return password;
+        }
+
         public FormValidation doCheckId(@QueryParameter String value) {
-            if (Util.fixEmpty(value) == null) {
+            if (Util.fixEmpty(value) == null)
                 return FormValidation.error(Messages.JDKInstaller_DescriptorImpl_doCheckId()); // improve message
-            } else {
-                // XXX further checks? 
-                return FormValidation.ok();
-            }
+            return FormValidation.ok();
         }
 
         /**
@@ -587,16 +624,28 @@ public class JDKInstaller extends ToolInstaller {
          * @return never null.
          */
         public List<JDKFamily> getInstallableJDKs() throws IOException {
-            return Arrays.asList(JDKList.all().get(JDKList.class).toList().jdks);
+            return Arrays.asList(JDKList.all().get(JDKList.class).toList().data);
         }
 
         public FormValidation doCheckAcceptLicense(@QueryParameter boolean value) {
+            if (username==null || password==null)
+                return FormValidation.errorWithMarkup(Messages.JDKInstaller_RequireOracleAccount(Stapler.getCurrentRequest().getContextPath()+'/'+getDescriptorUrl()+"/enterCredential"));
             if (value) {
                 return FormValidation.ok();
             } else {
                 return FormValidation.error(Messages.JDKInstaller_DescriptorImpl_doCheckAcceptLicense()); 
             }
         }
+
+        /**
+         * Submits the Oracle account username/password.
+         */
+        public HttpResponse doPostCredential(@QueryParameter String username, @QueryParameter String password) throws IOException, ServletException {
+            this.username = username;
+            this.password = Secret.fromString(password);
+            save();
+            return HttpResponses.redirectTo("credentialOK");
+        }
     }
 
     /**
